require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

describe Hermes::Api::Dmo::HousingRequestInitializersController, :type => :api do
  routes { Hermes::Engine.routes }
  include Devise::Test::ControllerHelpers

  let!(:admin) { create :administrator, auth_roles: ['superadministrateur'] }
  let!(:collab) { create(:user, auth_roles: ['collaborateur offre logement'])}
  let!(:demandeur) {create(:user)}
  let!(:other_user) {create(:user)}

  describe 'GET index' do
    let!(:hri1) do 
      create(
        :housing_request_initializer, 
        user: demandeur, 
        email: 'demandeur@mail.com',
        pin_code: "code",
        writers: [demandeur.id.to_s],
        hashed_pin_code: Digest::SHA2.new(512).hexdigest('code')
      )
    end 

    it 'as collab' do
      authenticated_header(request, collab)
      get 'index'
      expect(response).to have_http_status 200
      expect(json['data'].length).to eq 1
      expect(json['data'][0]['attributes']['pin_code']).to eq('code')
      expect(json['data'][0]['attributes']['email']).to eq('demandeur@mail.com')
    end

    it 'as creator' do 
      authenticated_header(request, demandeur)
      get 'index'
      expect(response).to have_http_status 200
      expect(json['data'].length).to eq 1
      expect(json['data'][0]['attributes']['pin_code']).to eq('unauthorized to see')
      expect(json['data'][0]['attributes']['email']).to eq('d*******r@mail.com')
    end 

    it 'as other user' do 
      authenticated_header(request, other_user)
      get 'index'
      expect(response).to have_http_status 200
      expect(json['data'].length).to eq 0
    end 

  end
end

require_relative '../../swagger_helper'

describe Hermes::Api::Dmo::HousingRequestInitializersController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  tags = 'API_DMO_HOUSING_REQUEST_INITIALIZERS'

  
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes
  let!(:superadmin) { create(:user, auth_roles: ['superadministrateur']) }

  path "/api/dmo/housing_request_initializers" do
    get "Retrieves all housing_request_initializer" do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      Hermes::Api::Dmo::HousingRequestInitializersController.acceptable_params.sort.each do |params_name, options|
        parameter name: params_name.to_s, in: options[:in] || :query, type: options[:type], required: options[:required] || false, description: options[:description] || ''
      end

      security [Bearer: {}]
      paginator

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }

        run_test!
      end 

      response '200', 'housing_request_initializer found' do
        let(:Authorization) { "Bearer #{token_for(superadmin)}" }
        let!(:housing_request_initializers) { create_list(:housing_request_initializer, 5) }

        include_context 'with integration test'
        include_context 'valid jsonapi collection' do
          let(:expected_collection) { housing_request_initializers }
        end

      end
    end
  end

  path '/api/dmo/housing_request_initializers/{id}' do
    get 'Retrieves a single housing_request_initializer by id' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]
      let(:housing_request_initializer) { create(:housing_request_initializer) }
      let(:id) {housing_request_initializer.id}

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        run_test!
      end 

      response '404', 'not found' do
        let(:Authorization) { "Bearer #{token_for(superadmin)}" }
        let(:id) { 1 }
        run_test!
      end 

      response '200', 'housing_request_initializer found' do
        let(:Authorization) {"Bearer #{token_for(superadmin)}"}

        include_context 'with integration test'
        include_context 'valid jsonapi ressource' do
          let(:expected_ressource) {housing_request_initializer}
        end
      end
    end
  end
end