require 'rails_helper'
require 'faker'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

UNIQUES_NUMBERS = ["9761220000236GDPUB","0110818044073AL004","0111018045296AL004"]
  
RSpec.describe Hermes::Api::Dmo::NudsController, :type => :api do
  routes {Hermes::Engine.routes}
  include Devise::Test::ControllerHelpers
  
  let!(:responsible) {create(:user, roles: ['Administrator'])}
  let!(:user) {create(:user)}
  let!(:user1) {create(:user, auth_roles: ['demandeur'])}
  #TESTS BASIQUES D'API
  
  describe 'POST /nuds' do
    describe 'Create nud with minimal valid datas' do
      let!(:applicant) {create(:applicant)}
      let!(:request1) {create(:housing_request, main_applicant: applicant)}
      before :each do
        drain_all_jobs
        authenticated_header(request, responsible)
        post 'create', {
          data: {
            attributes: {
              user_id: responsible.id.to_s,
              als_id: request1.reload.als_id.to_s,
              unique_number: "0110418041931AL004"
            }
          }
        }
      end
      it 'returns status code 201' do
        expect(response).to have_http_status(201)
        data = json['data']
        expect(data["attributes"]["provider"]).to eq("SNE")
      end
    end
    
    describe 'Create with alreay existing pair of uniq_nud and hosuing_request_id' do
      unique_number = "azdqddqzd"
      let!(:applicant) {create(:applicant)}
      let!(:request1) {create(:housing_request, main_applicant: applicant)}
      let!(:nud_test) {create(:nud, housing_request: request1, unique_number: unique_number)}
      before :each do
        drain_all_jobs
        authenticated_header(request, responsible)
        post 'create', {
          data: {
            attributes: {
              user_id: responsible.id.to_s,
              als_id: request1.reload.als_id.to_s,
              unique_number: unique_number,
              creation_date: '1867-03-24'
            }
          }
        }
      end
      
      it 'returns status code 201' do
        expect(response).to have_http_status(201)
      end
      
      it 'doesn\'t create another nud' do
        expect(Hermes::Nud.count).to be 1
      end
      
      it 'updates nud_test' do
        data = json['data']
        expect(data['id']).to eq nud_test.id.to_s
        expect(Date.parse(data['attributes']['creation_date'])).to eq(Date.parse('1867-03-24'))
      end
    end
    describe 'Create with alreay existing pair of uniq_nud and hosuing_request_id that is active' do
      unique_number = "azdqddqzd"
      let!(:applicant) {create(:applicant)}
      let!(:applicant2) {create(:applicant)}
      let!(:request1) {create(:housing_request, main_applicant: applicant, is_active: true)}
      let!(:request2) {create(:housing_request, main_applicant: applicant2, is_active: true)}
      let!(:nud_test) {create(:nud, housing_request: request1, unique_number: unique_number)}
      before :each do
    
        drain_all_jobs
        
        authenticated_header(request, responsible)
        post 'create', {
          data: {
            attributes: {
              user_id: responsible.id.to_s,
              als_id: request2.reload.als_id.to_s,
              unique_number: unique_number,
              creation_date: '1867-03-24'
            }
          }
        }
      end
      
      it 'returns status code 422' do
        expect(response).to have_http_status(422)
      end
      
      it 'doesn\'t create another nud' do
        expect(Hermes::Nud.count).to be 1
      end
      

    end

    #housing_request_id:'BSON::ObjectId',
    describe 'post nud with request' do
      let!(:applicant1) {create(:applicant)}
      let!(:request1) {create(:housing_request, main_applicant: applicant1)}
      before :each do
        drain_all_jobs
        authenticated_header(request, responsible)
        post 'create', {
          data: {
            attributes: {
              user_id: responsible.id.to_s,
              als_id: request1.reload.als_id.to_s,
            }
          }
        }
      end
      it 'returns status code 201 and the value appears in json' do
        expect(response).to have_http_status(201)
        expect(json['data']['attributes']['housing_request_id']).to eq(request1.id.to_s)
      end
    end
  end
  
  describe 'GET /nuds/pending_for_current_user' do 
    let!(:demandeur1) {create(:user)}
    let!(:demandeur2) {create(:user)}
    let!(:demandeur3) {create(:user)}
    let!(:other_user) {create(:user)}
    let!(:hri1) {create(:housing_request_initializer, email: "main_user@mail.com", user: demandeur1, done: false, pin_code_created_at: DateTime.now - 1.hours)}
    let!(:hri2) {create(:housing_request_initializer, user: demandeur2, done: true,  pin_code_created_at: DateTime.now - 1.hours)}
    let!(:hri3) {create(:housing_request_initializer, user: demandeur3, done: false,  pin_code_created_at: DateTime.now - 25.hours)}

    it 'get hri for demandeur1' do 
      authenticated_header(request, demandeur1)
      get 'pending_for_current_user'
      expect(response).to have_http_status(200)
      expect(json['data']['id']).to eq(hri1.id.to_s)
      expect(json['data']['attributes']['expired']).to eq(false)
      expect(json['data']['attributes']['email']).to eq("m*******r@mail.com")
    end 

    it 'not get hri for demandeur2' do 
      authenticated_header(request, demandeur2)
      get 'pending_for_current_user'
      expect(response).to have_http_status(404)
    end 

    it 'not get hri for other_user' do 
      authenticated_header(request, other_user)
      get 'pending_for_current_user'
      expect(response).to have_http_status(404)
    end 

    it 'too old' do 
      authenticated_header(request, demandeur3)
      get 'pending_for_current_user'
      expect(response).to have_http_status(200)
      expect(json['data']['attributes']['expired']).to eq(true)
    end 

  end 
  
  describe 'GET /nuds/:id' do
    describe 'verif it work' do
      let!(:applicant1) {create(:applicant)}
      let!(:request1) {create(:housing_request, main_applicant: applicant1)}
      let!(:nud1) {create(:nud, department: '24', user_id: responsible.id.to_s, housing_request: request1)}
      before :each do
        authenticated_header(request, responsible)
        get 'show', {id: nud1.id}
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
      end
    end
    describe 'verif it work not work with a non authorized user' do
      let!(:applicant1) {create(:applicant)}
      let!(:request1) {create(:housing_request, main_applicant: applicant1)}
      let!(:nud1) {create(:nud, department: '24', user_id: responsible.id.to_s, housing_request: request1)}
      
      before :each do
        authenticated_header(request, user)
        get 'show', {id: nud1.id}
      end
      it 'returns status code 404' do
        expect(response).to have_http_status(404)
      end
    end
  end
  
  describe 'authentify_with_pin_code' do 
    let!(:unique_number) { '0130821004617G3U05' }

    it 'with form errors' do 
      authenticated_header(request, responsible)
      post :send_pin_code_on_mail, id: unique_number 
      hri = Hermes::HousingRequestInitializer.find_by(unique_number: unique_number)
      post :authentify_with_pin_code, id: hri.unique_number, pin_code: hri.pin_code
      hr = Hermes::HousingRequest.find_by(is_active: true, unique_numbers: unique_number)
      expect(response.status).to eq(200)
      expect(hr.form_sne_validation_errors.empty?).to eq(false)
      expect(json['data']['attributes']['form_sne_validation_errors'].empty?).to eq(false)
    end 
  end 

  describe 'check_if_deleted_applicant_delete_ids' do 
    let!(:unique_number) { '1110421805072GDPUB' }

    it 'do_reset_applicant_used_for_siret' do 
      authenticated_header(request, responsible)
      post :send_pin_code_on_mail, id: unique_number 
      hri = Hermes::HousingRequestInitializer.find_by(unique_number: unique_number)
      post :authentify_with_pin_code, id: hri.unique_number, pin_code: hri.pin_code
      hr = Hermes::HousingRequest.find_by(is_active: true, unique_numbers: unique_number)
      expect(response.status).to eq(200)
      created_coapplicant_id = hr.coapplicants[0][:id]
      hr.applicant_used_for_siret= created_coapplicant_id
      hr.save 
      expect(hr.form_front_validation_errors.length).to eq(1)
      applicant_to_edit = Hermes::Applicant.find(created_coapplicant_id)
      applicant_to_edit.main_employer_name = "la poste"
      applicant_to_edit.save
      hr.save
      expect(hr.form_front_validation_errors).to eq([])
      expect(hr.applicant_used_for_siret_firstname).to eq("Ghizlane")
      hr.unique_numbers =["1110421805099GDPUB"]
      hr.update_request_from_sne(hr.user)
      hr.save
      expect(hr.applicant_used_for_siret).to eq(nil)
      expect(hr.applicant_used_for_siret_firstname).to eq("Rabie")
      expect(hr.form_front_validation_errors.length).to eq(1)
    end 
  end 
  describe 'check_if_case_insensitive' do 
    let!(:unique_number) { '1110421805072GDPUB' }

    it 'do_reset_applicant_used_for_siret' do 
      authenticated_header(request, responsible)
      post :send_pin_code_on_mail, id: unique_number 
      hri = Hermes::HousingRequestInitializer.find_by(unique_number: unique_number)
      post :authentify_with_pin_code, id: hri.unique_number, pin_code: hri.pin_code
      hr = Hermes::HousingRequest.find_by(is_active: true, unique_numbers: unique_number)
      expect(response.status).to eq(200)
      created_coapplicant_id = hr.coapplicants[0][:id]
      created_main_applicant_id = hr.main_applicant_id
      hr.unique_numbers =["1110421805073GDPUB"]
      hr.update_request_from_sne(hr.user)
      hr.save
      expect(hr.main_applicant_id).to eq(created_main_applicant_id)
      expect(hr.coapplicants[0][:id]).to eq(created_coapplicant_id)
    end 
  end 
end

RSpec.describe Hermes::Api::Dmo::NudsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  require_relative '../../swagger_helper'
  tags = 'API_DMO_NUDS'
  
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes
  let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
  let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}
  let!(:housing_request) { create(:active_request)}
  let!(:admin) { create(:user, roles: ['Administrator'])}

  path '/api/dmo/nuds' do
    get 'Retrieves all nuds' do
      tags tags
      produces 'application/json'
  
      Hermes::Api::Dmo::NudsController.acceptable_params.sort.each do |params_name, options|
        parameter name: params_name.to_s, in: options[:in] || :query, type: options[:type], required: options[:required] || false, description: options[:description] || ''
      end
  
      security [Bearer: {}]
      paginator
  
      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
  
        run_test!
      end 
  
      response '200', 'nuds found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let!(:nuds) { create_list(:nud, 5, housing_request_id: housing_request.id) }
  
        include_context 'with integration test'
        include_context 'valid jsonapi collection' do
          let(:expected_collection) { nuds }
        end
      end
    end
  end
  
  path '/api/dmo/nuds' do
    post 'Create nud' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      security [Bearer: {}]
      json_params = {}
      Hermes::Api::Dmo::NudsController.mutate_params.each do |key, options|
        json_params[key] = nice_type(options[:type])
      end
      parameter(
        name: :params,
        in: :body,
        schema: jsonify_params(json_params),
      )
      let(:params) do
        jsonapi_attributes(
          {  housing_request_id: housing_request.id }
        )
      end
  
      response '201', 'nud created' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
  
        include_context 'with integration test'
        include_context 'valid jsonapi'
        include_context 'not containings errors'
  
        after do |_example|
          json = JSON.parse(response.body)
  
          expect(json['data']).to be_a Hash
          expect(json['data']['id']).to_not be nil
          expect(json['data']['type']).to eq 'nuds'
        end
      end
    end
  end
  
  path '/api/dmo/nuds/{id}' do
    get 'Retrieves a single nud by id' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]
  
      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
        let(:id) { 1 }
  
        run_test!
      end 
  
      response '404', 'not found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { 1 }
  
        run_test!
      end 
      
      response '200', 'nud found' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}
        let(:nud) {create(:nud, housing_request_id: housing_request.id)}
        let(:id) {nud.id}
  
        include_context 'with integration test'
        include_context 'valid jsonapi ressource' do
          let(:expected_ressource) {nud}
        end
      end
    end
  end
  
  path '/api/dmo/nuds/{id}' do
    put 'Update nud' do
      tags tags
      produces 'application/json'
      consumes 'application/json'
      security [Bearer: {}]
      json_params = {}
      Hermes::Api::Dmo::NudsController.mutate_params.each do |key, options|
        json_params[key] = nice_type(options[:type])
      end
      parameter name: :id, in: :path, type: :string
      let(:id) { create(:nud, housing_request_id: housing_request.id, user: admin).id }
      parameter(
        name: :params,
        in: :body,
        schema: jsonify_params(json_params),
      )
      let(:params) do
        jsonapi_attributes(
          {  housing_request_id: housing_request.id, }
        )
      end
  
      response '200', 'nud updated' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
  
        include_context 'with integration test'
        include_context 'valid jsonapi'
        include_context 'not containings errors'
  
        after do |_example|
          json = JSON.parse(response.body)
          expect(json['data']).to be_a Hash
          expect(json['data']['id']).to_not be nil
          expect(json['data']['type']).to eq 'nuds'
        end
      end
    end
  end
  
  path '/api/dmo/nuds/{id}' do
    delete 'Delete a single nud by id' do
      tags tags
      produces 'application/json'
      parameter name: :id, in: :path, type: :string
      security [Bearer: {}]
  
      response '204', 'nud deleted' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let(:id) { create(:nud, housing_request_id: housing_request.id).id }
        run_test!
      end
    end
  end


  path '/api/dmo/nuds/{numero_unique_departemental}/send_pin_code_on_mail' do
    post 'send_pin_code_on_mail' do
      tags tags
      produces 'application/json'
      parameter name: :numero_unique_departemental, in: :path, type: :string, required: true
      security [Bearer: {}]
      
      response '200', 'pin code send on email' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}
        let(:numero_unique_departemental) {UNIQUES_NUMBERS[0]}
        
        run_test!
      
      end
    end
  end

  path '/api/dmo/nuds/{numero_unique_departemental}/send_new_pin_code' do
    post 'send_new_pin_code' do
      tags tags
      produces 'application/json'
      parameter name: :numero_unique_departemental, in: :path, type: :string, required: true
      security [Bearer: {}]
      
      response '200', 'pin code send on email' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}
        let(:numero_unique_departemental) {UNIQUES_NUMBERS[0]}
        
        run_test!
      
      end
    end
  end

  path '/api/dmo/nuds/{numero_unique_departemental}/authentify_with_pin_code' do
    post 'send_pin_code_on_mail' do
      tags tags
      produces 'application/json'
      parameter name: :numero_unique_departemental, in: :path, type: :string, required: true
      security [Bearer: {}]
      parameter name: :pin_code, in: :body, type: :string, requied: true
      response '422', 'unable to validate pin_code' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}
        let(:nud) {create(:nud, housing_request: housing_request, unique_number: "3REFLIDKFF")}
        let(:numero_unique_departemental) {nud.unique_number}
        let(:pin_code) {"slip"}
        run_test!
      end
    end
  end

  path '/api/dmo/nuds/sne_department_list' do
    get 'sne_department_list' do
      tags tags
      produces 'application/json'
      security [Bearer: {}]
      
      response '200', 'retrive the list of allowed departments for SNE' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}
        run_test!
      end
    end
  end

  path '/api/dmo/nuds/imhoweb_department_list' do
    get 'imhoweb_department_list' do
      tags tags
      produces 'application/json'
      security [Bearer: {}]
      
      response '200', 'retrive the list of allowed departments IMHOWEB' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}
        run_test!
      end
    end
  end

  path '/api/dmo/nuds/pending_for_current_user' do
    get 'Check if an nud is pending for user' do
      tags tags
      produces 'application/json'
      security [Bearer: {}]
  
      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }
  
        run_test!
      end 
  
      response '404', 'not found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
  
        run_test!
      end 
      
      response '200', 'nud found' do
        let(:Authorization) {"Bearer #{token_for(admin)}"}
        let!(:nud) {create(:housing_request_initializer, housing_request_id: housing_request.id, user_id: admin.id, done: false, pin_code_created_at: Time.now.utc)}
        
        run_test!
        after do |_example|
          json = JSON.parse(response.body)
          expect(json['data']).to be_a Hash
          expect(json['data']['id']).to_not be nil
          expect(json['data']['type']).to eq 'housing_request_initializers'
        end
      end
    end
  end
end
